#!/usr/bin/perl -w
#
# $Id: notify.pl,v 1.14 2005/05/03 10:00:31 peter Exp $
#
# notify.pl - Notification script for OP5 Monitor
#
# Author: Andreas Ericsson <ae@op5.se>
#
# Copyright(C) 2004 OP5 AB
# All rights reserved.
#

use POSIX;
use strict;
use File::Glob ':glob';

# Http-post addition  
use HTTP::Request; ####
use LWP::UserAgent; ###
use HTTP::Request::Common; ###

our $url_base = "https";
if ( -e "/opt/monitor/op5/notify/notify.inc.pl" ) {
	require "/opt/monitor/op5/notify/notify.inc.pl";
} 

# set some defaults.
my ($sec, $min, $hour, $mday, $month, $year, $wday, $yday, $is_dst);
my $i;
my ($contact_pager, $contact_email, $host, $skin_file, $contact_name, $service, $notification_type, $contact_httppost);
my (%val, $k, $v, $from_email);
my $skin_dir = '/opt/monitor/op5/notify/';
my $custom_skin_dir = '/opt/monitor/op5/notify/custom-skins/';
my ($dir, $file);
my $debug = 0;	# set to 1 for debug output
my $x_var_dir = '/opt/monitor/op5/notify/extra_host_vars';
my @x_var_files;
my %x_vars;
my $foo;
my $result;
my $now;

sub email_notify();
sub pager_notify();
sub httppost_notify(); # http-post addition 
sub usage();

# --- Functions (ok subprocedures then, you silly, silly language)
$SIG{'ALRM'} = sub {
	print("Notification timed out\n");
	exit(1);
};

sub pager_notify() {
	my $buf = undef;
	my $dir = $skin_dir . 'skins.sms';
    if (-d "$custom_skin_dir/skins.sms") {
		$dir = "$custom_skin_dir/skins.sms";
    }
	if(-d "$dir/$contact_name") {
		$dir = "$dir/$contact_name";
	}
	my $file = undef;
	my $pager = '/usr/bin/smssend';

	if(!-x $pager) {
		$pager = '/usr/bin/sendsms';
	}

	if($debug) {
		open(FILE, ">> /tmp/debug.pager_notify");
		print FILE "using $pager to send sms. dir is $dir\n";
		print FILE "skin_file is $skin_file\n";
		print FILE "looking for $dir/$skin_file\n";
	}

	# get the skin-file to use, but only if it exists in the right directory
	if($service) {
		$file = "$dir/service.default";
	}
	else {
		$file = "$dir/host.default";
	}
	if(-f "$dir/$skin_file" || -l "$dir/$skin_file") {
		$file = "$dir/$skin_file";
	}

	$debug and print FILE "skinfile is $file\n";
	if ($debug and !$file) {
		print FILE "pager_notify(): No suitable skin file found\n";
	}
	if(!$file) {
		if($debug) {
			print(FILE "skin-file not set. returning\n");
			close(FILE);
		}
		return;
	}
	$debug and print FILE "pager_notify(): using file $file\n";
	-r $file or print FILE "pager_notify(): skin file $file not readable\n";
	if(!-r $file) {
		$debug and print "pager_notify(): skin file $file not readable\n";
		print(FILE "pager_notify(): skin-file $file not readable. returning\n");
		close(FILE);
		return;
	}

	$buf .= `cat $file 2> /dev/null`;
	$? != 0 and return;

	foreach $k (keys %val) {
		$buf =~ s/#$k#/$val{$k}/g;
	}

	open(PIPE, "| $pager $contact_pager");
	print(PIPE "$buf\n");
	close(PIPE);

	if($debug) {
		open(FILE, ">> /tmp/pager.log");
		print(FILE "$buf\n-------------------------------\n$file\n===========================\n");
		close(FILE);
	}
};

# send an email, using the mail (or mailx command)
sub email_notify() {
	my $buf = undef;
	my $dir = $skin_dir . 'skins.mail';
	if (-d "$custom_skin_dir/skins.mail") {
	    $dir = "$custom_skin_dir/skins.mail";
	}
	my $file = undef;
	my $hostname;
	my $cgi_cfg = '/opt/monitor/etc/cgi.cfg';
	my $html_path = "/monitor";;
	my $status_url = "$url_base://";
	my $mailer = '/usr/sbin/sendmail -t -i';
	my $ack_url;

	$from_email and $mailer .= " -f$from_email";

	# set up the status URL
	$hostname = `hostname -f` or $hostname = `hostname`;
	chomp($hostname);
	$status_url .= $hostname;
	$val{"SERVERADDRESS"} = $status_url;

	$status_url .= $html_path;

#	$status_url .= "/cgi-bin/status.cgi?host=$host";
	$status_url .= "/index.php/status/service/$host";
	$val{"STATUS_URL"} = $status_url;

	$ack_url = "$url_base://" . $hostname . $html_path;
#	$ack_url .= '/cgi-bin/cmd.cgi?cmd_typ=';
	$ack_url .= '/index.php/command/submit?cmd_typ=';
#	https://dev-mon.int.op5.se/ninja/index.php/command/submit?cmd_typ=ACKNOWLEDGE_SVC_PROBLEM&host_name=build-centos-x86_64&service=Disk+usage+%2F
	if ($service) {
#		$ack_url .= '34&host=' . $host . '&service=' . $service;
		$ack_url .= 'ACKNOWLEDGE_SVC_PROBLEM&host_name=' . $host . '&service=' . $service;
	}
	else {
#		$ack_url .= '33&host=' . $host;
		$ack_url .= 'ACKNOWLEDGE_HOST_PROBLEM&host_name=' . $host;
	}
	$ack_url =~ s/([^-+=_A-Za-z0-9\/:.?&])/sprintf("%%%02X", ord($1))/seg;
	$val{"ACK_URL"} = $ack_url;

	if($debug) {
		open(FILE, ">> /tmp/debug.email_notify");
		print FILE "using $mailer to send notifications. dir is $dir\n";
		print FILE "skin_file is $skin_file\n";
		print FILE "looking for $dir/$skin_file\n";
		print FILE "Using $mailer to send mail\n";
		print FILE "status_url = $status_url\n";
	}


	# -- set up the buffer
	$buf = '';

	# get the skin-file to use. If a user doesn't want notifications
	# of a particular type, all they have to do is to create a separate
	# directory named $skin_dir/skins.mail/$contact_name and simply skip
	# copying the files they don't want notifications for.
	if(-d "$dir/$contact_name") {
		$dir = "$dir/$contact_name";
	}
	$file = "$dir/host.default";
	$service and $file = "$dir/service.default";
	if(-f "$dir/$skin_file" || -l "$dir/$skin_file") {
		$file = "$dir/$skin_file";
	}

	if(!(-f "$file") and !(-l "$file")) {
		undef($file);
	}
	if(!$file) {
		if($debug) {
			print FILE "\$file not set, or not a file or symlink. returning\n";
			close(FILE);
		}
		return;
	}

	$buf = `cat $file 2> /dev/null`;
	if($? != 0) {
		if($debug) {
			print(FILE "skin file ($file) not readable. $!\n");
			close(FILE);
		}
		return;
	}

	# read extra vars, if any
	my $x_vars = '';
	if(-d "$x_var_dir/$contact_name") {
		chdir("$x_var_dir/$contact_name");
		@x_var_files = glob('*');
		$debug and print $#x_var_files + 1 . " extra vars found in $x_var_dir/$contact_name\n";
		if($#x_var_files != -1) {
			for($i=0; $i<=$#x_var_files; $i++) {
				$debug and print "$i = $x_var_files[$i]\n";
				$foo = `grep ^$host $x_var_files[$i] | sed 's/^.*://'` and
				  $x_vars .= $x_var_files[$i] . '=' . $foo;
			}
		}
	}
	$val{"extra_host_vars"} = $x_vars;

	# -- get status url

	foreach $k (keys %val) {
		$buf =~ s/#$k#/$val{$k}/g;
	}

	$debug and print FILE "$buf\n";
	# everything is set, so pipe to the mailer program with extra newline
	open(PIPE, "| $mailer");
	print(PIPE "$buf\n");
	close(PIPE);

	if($debug) {
		close(FILE);
		open(FILE, ">> /tmp/email.log");
		print(FILE "$buf\n-------------------------------\n$file\n=====================================\n");
		close(FILE);
	}
};


# Post xml stuff to web server
sub httppost_notify() {
	my $buf = undef;
	my $dir = $skin_dir . 'skins.htmlpost';
	if (-d "$custom_skin_dir/skins.htmlpost") {
	    $dir = "$custom_skin_dir/skins.htmlpost";
	}
	my $file = undef;
	my $hostname;
	my $cgi_cfg = '/opt/monitor/etc/cgi.cfg';
	my $html_path = "/monitor";;
	my $status_url = "$url_base://";
	#   my $mailer = '/usr/sbin/sendmail -t -i';
	
	# set up the status URL
	$hostname = `hostname -f` or $hostname = `hostname`;
	chomp($hostname);
	$status_url .= $hostname;
	
	$status_url .= $html_path;
	
	$status_url .= "/cgi-bin/status.cgi?host=$host";
	$val{"STATUS_URL"} = $status_url;
	
	if($debug) {
		close(FILE);
		open(FILE, ">> /tmp/debug.httppost_notify");
		print FILE "dir is $dir\n";
		print FILE "skin_file is $skin_file\n";
		print FILE "looking for $dir/$skin_file\n";
		print FILE "status_url = $status_url\n";
	}
	
	
	# -- set up the buffer
	$buf = '';
	
	# get the skin-file to use. If a user doesn't want notifications
	# of a particular type, all they have to do is to create a separate
	# directory named $skin_dir/skins.mail/$contact_name and simply skip
	# copying the files they don't want notifications for.
    if(-d "$dir/$contact_name") {
		$dir = "$dir/$contact_name";
	}
	
	$file = "$dir/host.default";
	$service and $file = "$dir/service.default";
	if(-f "$dir/$skin_file" || -l "$dir/$skin_file") {
		$file = "$dir/$skin_file";
	}
	
	if(!(-f "$file") and !(-l "$file")) {
		undef($file);
	}
	if(!$file) {
		if($debug) {
			print FILE "$file not set, or not a file or symlink. returning\n";
			close(FILE);
		}
		return;
	}
	
	
	$buf = `cat $file 2> /dev/null`;
	if($? != 0) {
		if($debug) {
			print(FILE "skin file ($file) not readable. $!\n");
			close(FILE);
		}
		return;
	}
	
	
	# read extra vars, if any
	my $x_vars = '';
	if(-d "$x_var_dir/$contact_name") {
		chdir("$x_var_dir/$contact_name");
		@x_var_files = glob('*');
		$debug and print $#x_var_files + 1 .
			" extra vars found in $x_var_dir/$contact_name\n";
		if($#x_var_files != -1) {
			for($i=0; $i<=$#x_var_files; $i++) {
				$debug and print "$i = $x_var_files[$i]\n";
				$foo = `grep ^$host $x_var_files[$i] | sed 's/^.*://'` and
				$x_vars .= $x_var_files[$i] . '=' . $foo;
			}
		}
	}
	$val{"extra_host_vars"} = $x_vars;
	
	
	# -- get status url
	
	foreach $k (keys %val) {
		$buf =~ s/#$k#/$val{$k}/g;
	}
	
	$debug and print FILE "$buf\n";
	# everything is set, so pipe to the mailer program with extra newline
	
    if($debug) {
		close(FILE);
		open(FILE, ">> /tmp/httppost.log");
		print(FILE "\n====================================\nPosting xml to: $contact_httppost\n------------------------------------\n");
        print(FILE "$buf\n-----------------------------------\n");
        close(FILE);
    }
    my $contact_httppost = "" . $contact_httppost;
    my $content = $buf;

    my $header = HTTP::Headers->new(Content_Type=>'text/xml', 
		                                    Content_Length=>length($content));
    my $request = HTTP::Request->new('POST', $contact_httppost, 
		                                     $header, $content);


    my $ua = LWP::UserAgent->new;

    my $response = $ua->request($request);

    if($debug) {
		open(FILE, ">> /tmp/httppost.log");
		if($response->is_success){
			print(FILE "\nPost successfull.\nResponse from webserver: $response->{_content}");
		} else {
			print(FILE "\nPost failed.\nResponse from webserver: $response->{_content}");
		}
    }
};



sub usage() {
	print("Usage: notify.pl options variables...\n");
	print("\nWhere options are any or all of the following ('*' = required)\n\n");
	print("* -c | --contact  short contactname\n");
	print("  -m | --mail     contact email address. If blank no mail is sent\n");
	print("  -p | --pager    contact pager number. If blank no sms is sent\n");
    print("  -g | --httppost post xml data to http server. If blank no post is done\n");	
	print("* -h | --host     host this notification pertains to\n");
	print("  -s | --service  service_description (MUST be set for service notifications)\n");
	print("* -f | --skin     skin file (usually \$NOTIFICATIONTYPE\$)\n");
	print("  -d | --debug    testing mode. Print some debug info\n");
	print("  -F | --from     set 'From: ' email address on mail notifications\n");
	print("\n");
	print("- Freeform variables should be specified as 'VARIABLE=VALUE'.\n");
	print("- Variables mentioned in the skin file as #VARIABLE# will be\n");
	print("  replaced with its corresponding VALUE.\n");
	print("- The reserved variables HOSTNAME and STATUS_URL are always available.\n");
	print("- The reserved variable SERVICEDESC is available if -s is non-empty\n");
	print("- Unix timestamps past the unix billenium will be translated to ISO 8601 format.\n");
	print "\n";
	exit(1);
};

# --- main code


$now = time();
`echo $now >> /tmp/notify_timestamps`;

# parse arguments
if($#ARGV < 2) {
	usage();
}

$val{"FROMEMAIL"} = 'op5Monitor';
for($i=0; $i<=$#ARGV; $i++) {
	if($ARGV[$i] =~ /^-m|^--mail|^--email/) {
		$contact_email = $ARGV[++$i];
		$val{"CONTACTEMAIL"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^-F|^--from|^--from-email/) {
		$from_email = $ARGV[++$i];
		$val{"FROMEMAIL"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^-p|^--page/) {
		$contact_pager = $ARGV[++$i];
		$val{"CONTACTPAGER"} = $ARGV[$i];
	}
    elsif($ARGV[$i] =~ /^-g|^--httppost/) {
		$contact_httppost = $ARGV[++$i];
		$val{"CONTACTHTMLPOST"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^-h|^--host/) {
		$host = $ARGV[++$i];
		$val{"HOSTNAME"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^-u|^--user|^-c|^--contact/) {
		$contact_name = $ARGV[++$i];
		$val{"CONTACTNAME"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^-f|^--skin/) {
		$skin_file = $ARGV[++$i];
		$notification_type = $ARGV[$i];
		$val{"NOTIFICATIONTYPE"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^-d|^--debug/) {
		$debug = 1;
	}
	elsif($ARGV[$i] =~ /^-s|^--service/) {
		$service = $ARGV[++$i];
		$val{"SERVICEDESC"} = $ARGV[$i];
	}
	elsif($ARGV[$i] =~ /^(CONTACTADDRESS[0-6])=(.*)$/) {
		# we special-case CONTACTADDRESS macros, because they
		# frequently contain phonenumbers that invariably are
		# large enough to trigger the "assume timestamp" code
		$k = $1;
		$v = $2;
		$val{$k} = $v;
	}
	elsif($ARGV[$i] =~ /^([^=]*)=(.*)$/) {
		# this is a free form variable, so register it
		$k = $1;
		$v = $2;
		if($v && $v =~ /^\d*$/ && $v > 1000000000) {
			# number past the unix billenia. Assume timestamp and translate
			($sec, $min, $hour, $mday, $month, $year, $wday, $yday, $is_dst) = localtime($v);
			$v = sprintf("%4d-%02d-%02d %02d:%02d:%02d",$year+1900,$month+1,$mday,$hour,$min,$sec);
		}
		$val{$k} = $v;
	}
	else {
		print "Unknown option: $i" . $ARGV[$i] . "\n";
		usage();
	}
}

alarm(3);

$contact_name or $contact_name = "NO CONTACT DEFINED";
if($debug) {
	open(DEBUG_FILE, ">> /tmp/foo");
	foreach $k (keys %val) {
		print(DEBUG_FILE "$k=" . $val{$k} . "\n");
	}
}

$host or die("Must set host parameter");

# use a proper skin-file
if($service) {
	$skin_file = "service.$skin_file";
}
else {
	$skin_file = "host.$skin_file";
}

# $debug and $skin_file = 'debug';

# -- take action based on means of transportations supplied
if($contact_pager && $contact_pager =~ /.*\d*.*/) {
	$debug and print(DEBUG_FILE "Sending pager notification to $contact_pager\n");
	pager_notify();
}
else {
	$debug and print(DEBUG_FILE "NOT sending pager notification\n");
}

if($contact_email && $contact_email =~ /^.*$/) {
	$debug and print(DEBUG_FILE "Sending email notification to $contact_email\n");
	email_notify();
}
else {
	$debug and print(DEBUG_FILE "NOT sending EMAIL notification\n");
}

if($contact_httppost && $contact_httppost =~ /^.*$/) {
	$debug and print(DEBUG_FILE "Posting notification to $contact_httppost\n");
	httppost_notify();
}
else {
	$debug and print(DEBUG_FILE "NOT posting xml notification\n");
}

if($debug) {
	print(DEBUG_FILE "------------------------------------------------------\n");
	close(DEBUG_FILE);
}
